###########################################################################################################################
## Chernozhukov, Fernandez-Val, Hahn, and Newey: "Average and Quantile Effects in Nonseparable Panel Models"
## R program to generate Figure 1
###########################################################################################################################

##  MODEL: y_{t} = 1{y_{t-1}*gamma_{0} + alpha - epsilon_{t} > 0}, epsilon ~ F, y_{0} ~ Be(p)
##  alpha ~ N(0,1) 
##  AVERAGE MARGINAL EFFECTS: MU_{0} = E_{(y_{0}} [E_{alpha} [F(gamma_{0} + alpha) - F(alpha) | y_{0}]]
##  the number of periods including the initial condition is T+1

# this version: 08/28/2018


#####################################################
# PART I: PRELIMINARY FUNCTIONS
#####################################################

rm(list = ls());

# Function to compute integrals of the shifted normal

snorm	<- function(alpha, nu, a) {
	return(pnorm(nu + alpha)^a * dnorm(alpha));
	};



## Width of identification region


width_bound <- function(gamma, py0, T)
	
	{
	return( (1-py0)/T + py0*integrate(snorm, -Inf, Inf, nu = gamma, a = T-1)$value );
	};






#####################################################
# PART II: MAIN PROGRAM
#####################################################

########################
###### py0 = 1/2
########################




library(stats);

# Parameters of the problem

py0     <- 1/2;
TT      <- 2;

Ts	<- c(2,4,8,16,32,64);
gammas0 <- c(-50:50)/25;

# Initialization of variables

width_np  	<- matrix(0, nrow = length(gammas0), ncol = length(Ts));

# Main loop;

for (j in 1:length(Ts)) {;

	TT	<- Ts[j];
	for (i in 1:length(gammas0)) {;
		width_np[i,j]	<- width_bound(gammas0[i],  py0, TT);
		};
	};


postscript("width-bounds-dynamic-probit.eps", horizontal=FALSE, onefile=FALSE,height=8,width=6,pointsize=10);

plot( c(min(gammas0), max(gammas0)), c(0,.7), type="n",xlab=expression(paste(beta, '*')), ylab="Width bounds ATE", main="");

	for (j in 1:length(Ts)) {; 
	        lines(gammas0, width_np[,j], col = 1, lty = j, lwd = 1.5);
		};


	legend(min(gammas0),.7, c("T = 2", "T = 4", "T = 8", "T = 16", "T = 32", "T = 64"), lwd = 1, bty="n", lty= c(1:length(Ts)));


dev.off();
