###########################################################################################################################
## Chernozhukov, Fernandez-Val, Hahn, and Newey: "Average and Quantile Effects in Nonseparable Panel Models"
## R program to generate Figures A1 & A2
###########################################################################################################################

##  MODEL: y_{t} = 1{x_{t}*gamma_{0} + alpha - epsilon_{t} > 0}, epsilon ~ F, alpha ~ N(0,1), x_{t} = 1{nu + alpha - upsilon_{t} > 0}, upsilon ~ N(0,1)
##  AVERAGE MARGINAL EFFECTS: MU_{0} = E[F(gamma_{0} + alpha) - F(alpha)]


# This version: 08/28/2012


rm(list = ls());


#####################################################
# PART I: FUNCTIONS
#####################################################

# Funtion to compute marginal effects

meffect	<- function(alpha, gamma) {
	return((plogis(gamma + alpha, location = 0, scale = sqrt(3)/pi) - plogis(alpha, location = 0, scale = sqrt(3)/pi))*dnorm(alpha));
	}

meffectx0	<- function(alpha, gamma, T) {;
	return((plogis(gamma + alpha, location = 0, scale = sqrt(3)/pi) - plogis(alpha, location = 0, scale = sqrt(3)/pi)) * pnorm(-alpha)^T * dnorm(alpha));
	};

meffectx1	<- function(alpha, gamma, T) {;
	return((plogis(gamma + alpha, location = 0, scale = sqrt(3)/pi) - plogis(alpha, location = 0, scale = sqrt(3)/pi)) * pnorm(alpha)^T * dnorm(alpha));
	};

# Function to compute integrals of the shifted logis

slogis	<- function(alpha, nu) {
	return(plogis(nu + alpha, location = 0, scale = sqrt(3)/pi)*dnorm(alpha));
	};

# Function to compute probabilities of x sequences

probx	<- function(alpha, T, vecx) {;
	return( pnorm(alpha)^sum(vecx) * pnorm(-alpha)^(T - sum(vecx)) * dnorm(alpha) );
	};

# Function to compute true choice probabilities

proby	<- function(alpha, T, vecx, vecy, beta) {;
	return( prod( pnorm(beta*vecx + alpha)*vecy +  pnorm(- beta*vecx - alpha)*(1 - vecy) ) * dnorm(alpha)  );
	};


py00x00	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py01x00	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py10x00	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py11x00	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py00x01	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py01x01	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py10x01	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py11x01	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py00x10	<- function(alpha, beta) {;
	return(plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py01x10	<- function(alpha, beta) {;
	return(plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py10x10	<- function(alpha, beta) {;
	return(plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py11x10	<- function(alpha, beta) {;
	return(plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py00x11	<- function(alpha, beta) {;
	return(plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py01x11	<- function(alpha, beta) {;
	return(plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py10x11	<- function(alpha, beta) {;
	return(plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py11x11	<- function(alpha, beta) {;
	return(plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py000x000	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py001x000	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py010x000	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py100x000	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py011x000	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py101x000	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py110x000	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py111x000	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(-alpha)*dnorm(alpha));	
	};

py000x001	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py001x001	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py010x001	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py100x001	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py011x001	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py101x001	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py110x001	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py111x001	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py000x010	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py001x010	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py010x010	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py100x010	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py011x010	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py101x010	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py110x010	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py111x010	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py000x100	<- function(alpha, beta) {;
	return(plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py001x100	<- function(alpha, beta) {;
	return(plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py010x100	<- function(alpha, beta) {;
	return(plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py100x100	<- function(alpha, beta) {;
	return(plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py011x100	<- function(alpha, beta) {;
	return(plogis(-beta-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py101x100	<- function(alpha, beta) {;
	return(plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py110x100	<- function(alpha, beta) {;
	return(plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py111x100	<- function(alpha, beta) {;
	return(plogis(beta+alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(-alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py000x011	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py001x011	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py010x011	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py100x011	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py011x011	<- function(alpha, beta) {;
	return(plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py101x011	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py110x011	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py111x011	<- function(alpha, beta) {;
	return(plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py000x101	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py001x101	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py010x101	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py100x101	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py011x101	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py101x101	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py110x101	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py111x101	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py000x110	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py001x110	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py010x110	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py100x110	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py011x110	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py101x110	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py110x110	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py111x110	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha, location = 0, scale = sqrt(3)/pi)*pnorm(-alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py000x111	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py001x111	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py010x111	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py100x111	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py011x111	<- function(alpha, beta) {;
	return(plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py101x111	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py110x111	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(-alpha-beta, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};

py111x111	<- function(alpha, beta) {;
	return(plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*plogis(alpha+beta, location = 0, scale = sqrt(3)/pi)*pnorm(alpha)*pnorm(alpha)*pnorm(alpha)*dnorm(alpha));	
	};


## PAYm calculates the true probabilities using the model 

PAYm <- function(AY, x, gamma, alphas, rhos)
	{
	nAY   <- nrow(AY);
	T     <- ncol(AY);
	pAY   <- matrix(0,nAY,1);
	M     <- length(alphas);
	for (i in 1:nAY) {;
		aux <- matrix(0,M,T);
		for (t in 1:T) {;
			aux[,t] <- (plogis(gamma*x[1,t]  + alphas, location = 0, scale = sqrt(3)/pi))^AY[i,t] * (1 - plogis(gamma*x[1,t]  + alphas, location = 0, scale = sqrt(3)/pi))^(1 - AY[i,t]);
			};
		pAY[i,1] <- sum(rhos * apply(aux,1,prod));
		};
	return(pAY);
	};


## PIA calculates the conditional probabilities estimated by the model 

PIAY <- function(AY, x, gamma, alphas)
	{
	nAX   <- nrow(AX);
	nAY   <- nrow(AY);
	T     <- ncol(AY);
	M     <- length(alphas);
	PIAY  <- matrix(0,nAY, M);
	for (i in 1:nAY) {;
                aux <- matrix(0,M,T);
		for (t in 1:T) {;
			aux[,t] <- (plogis(gamma*x[1,t] + alphas, location = 0, scale = sqrt(3)/pi))^AY[i,t] * (1 - plogis(gamma*x[1,t] + alphas, location = 0, scale = sqrt(3)/pi))^(1 - AY[i,t]);
			};
		PIAY[i, ] <- apply(aux,1,prod);
		};
	return(PIAY);
	};


# ME calculates individual marginal effects

ME <- function(gamma, alphas, x)
	
	{
	T	<- ncol(x);
	return(plogis(gamma +  alphas, location = 0, scale = sqrt(3)/pi) - plogis(alphas, location = 0, scale = sqrt(3)/pi));
	};


#####################################################
# PART II: MAIN PROGRAM
#####################################################


library(stats);
library(lpSolve);
library(quadprog);

set.seed(888);

##############;
# T = 2;
##############;


# PARAMETERS FOR SIMULATION;

gammas0	<- 1;
nu	<- 0;
T	<- 2;
N	<- 5000000;
nu	<- 0;


alpha	<- sort(rnorm(N));
A	<- t(cbind(c(0,0), c(1,0), c(0,1), c(1,1)));	
AY      <- A;
AX      <- A;
nAY	<- nrow(AY);
nAX  	<- nrow(AX);


pAX	<- NULL;

for (i in 1:nAX) {;
	pAX	<- rbind(pAX, integrate(probx, -Inf, Inf, T = T, vecx = AX[i, ])$value);
	};


alpha0s	<- c(-Inf,4*(-10:10)/10,Inf);
alpha0s	<- c(-Inf,2*(-20:20)/10,Inf);
alpha0s <- qlogis(c(0:50)/50, location = 0, scale = sqrt(3)/pi);
alpha0s <- qlogis(c(0:100)/100, location = 0, scale = sqrt(3)/pi);
alphas 	<- c(-Inf,2*(-80:80)/10,Inf);
alphas 	<- qlogis(c(0:150)/150, location = 0, scale = sqrt(3)/pi);
alphas  <- qlogis(c(0:100)/100, location = 0, scale = sqrt(3)/pi);
M      	<- length(alphas);
A	<- t(cbind(c(0,0), c(1,0), c(0,1), c(1,1)));	
AY      <- A;
AX      <- A;
nAY	<- nrow(AY);
nAX  	<- nrow(AX);

# Parameters of the optimization

gridgamma0 <- 201;
ugamma0    <- (gridgamma0 - 1)/2;
gammas0    <- 2*2*(-ugamma0:ugamma0)/(gridgamma0-1);

gridgamma <- 201;
wgamma    <- .5;
wgrid     <- (gridgamma - 1)/2;


# Initialization of variables

OBJ    		<- matrix(-999, gridgamma0, gridgamma);

meffs00    	<- NULL;
meffs0    	<- NULL;
meffs0_x00	<- NULL;
meffs0_x01	<- NULL;
meffs0_x10	<- NULL;
meffs0_x11	<- NULL;

imeffs0_x01	<- NULL;
imeffs0_x10	<- NULL;
imeffs0_x00	<- NULL;
imeffs0_x11	<- NULL;
py_x00		<- NULL;
py_x11		<- NULL;


meffs_sp    	<- matrix(-999, gridgamma0, gridgamma);

umeffs_x00_sp <- matrix(-999, gridgamma0, gridgamma);
umeffs_x11_sp <- matrix(-999, gridgamma0, gridgamma);

lmeffs_x00_sp <- matrix(999, gridgamma0, gridgamma);
lmeffs_x11_sp <- matrix(999, gridgamma0, gridgamma);

gammas1	<- NULL;

f.obj 	<- t(cbind(t(rep(1,nAY*nAX+nAX)),t(rep(0,nAX*M))));
f.dir 	<- t(cbind(t(rep("=",nAY*nAX+nAX)),t(rep(">=", nAX*(nAY+1+M)))));
f_x.dir <- t(cbind(t(rep("=",nAY+1)),t(rep(">=", M))));

lnosol 	<- 0;
unosol 	<- 0;

lnosol_x00 <- 0;
unosol_x00 <- 0;
lnosol_x11 <- 0;
unosol_x11 <- 0;

for (i in 1:gridgamma0) {;
	gamma0 <- gammas0[i];
	lgamma <- wgrid * (gamma0 - wgamma) / wgamma;
	ugamma <- wgrid * (gamma0 + wgamma) / wgamma;
	gammas <- wgamma * (lgamma:ugamma) / wgrid;

	# TRUE CHOICE PROBABILITIES;

	pAY_x00	<- rbind(integrate(py00x00, -Inf, Inf, beta=gamma0)$value, integrate(py10x00, -Inf, Inf, beta=gamma0)$value, integrate(py01x00, -Inf, Inf, beta=gamma0)$value, integrate(py11x00, -Inf, Inf, beta=gamma0)$value)/pAX[1,];
	pAY_x01	<- rbind(integrate(py00x01, -Inf, Inf, beta=gamma0)$value, integrate(py10x01, -Inf, Inf, beta=gamma0)$value, integrate(py01x01, -Inf, Inf, beta=gamma0)$value, integrate(py11x01, -Inf, Inf, beta=gamma0)$value)/pAX[3,];
	pAY_x10	<- rbind(integrate(py00x10, -Inf, Inf, beta=gamma0)$value, integrate(py10x10, -Inf, Inf, beta=gamma0)$value, integrate(py01x10, -Inf, Inf, beta=gamma0)$value, integrate(py11x10, -Inf, Inf, beta=gamma0)$value)/pAX[2,];
	pAY_x11	<- rbind(integrate(py00x11, -Inf, Inf, beta=gamma0)$value, integrate(py10x11, -Inf, Inf, beta=gamma0)$value, integrate(py01x11, -Inf, Inf, beta=gamma0)$value, integrate(py11x11, -Inf, Inf, beta=gamma0)$value)/pAX[4,];

	# TRUE AVERAGE EFFECT;
	meffs00[i]	<- integrate(meffect, -Inf, Inf, gamma=gamma0)$value;
	meffs0_x01[i] 	<- pAY_x01[3,1] - pAY_x01[2,1];
	meffs0_x10[i] 	<- pAY_x10[2,1] - pAY_x10[3,1];
	meffs0_x00[i] 	<- integrate(meffectx0, -Inf, Inf, gamma=gamma0, T=T)$value / pAX[1,];
	meffs0_x11[i] 	<- integrate(meffectx1, -Inf, Inf, gamma=gamma0, T=T)$value / pAX[nAX,];


	# BOUNDS FOR THE PARAMETER;

	# QUADRATIC PROGRAM: REGULARIZATION OF PROBABILITIES TO THE MODEL;

	lambda		<- 1/(N*log(N));

	alpha0s_x00	<- alpha0s;
	alpha0s_x01	<- alpha0s;
	alpha0s_x10	<- alpha0s;
	alpha0s_x11	<- alpha0s;

	MM_x00		<- length(alpha0s);
	MM_x01	   	<- length(alpha0s);
	MM_x10	   	<- length(alpha0s);
	MM_x11	   	<- length(alpha0s);

	piAY_x00	<- PIAY(AY, t(AX[1,]), gamma0, alpha0s);
	piAY_x01  	<- PIAY(AY, t(AX[3,]), gamma0, alpha0s);
	piAY_x10  	<- PIAY(AY, t(AX[2,]), gamma0, alpha0s);
	piAY_x11  	<- PIAY(AY, t(AX[4,]), gamma0, alpha0s);

	aa1_x00    	<- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11 ));
	aa1_x01    	<- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00), (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01))), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    	<- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))), matrix(0, nAY, MM_x11));
	aa1_x11    	<- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );

	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- as.real(QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);


	#REGULARIZED PROBABILITIES;
	pAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	pAY_x01     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	pAY_x10     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	pAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));

	# NONPARARAMETRICALLY IDENTIFIED EFFECTS AND BOUNDS;
	imeffs0_x01[i] 	<- pAY_x01[3,1] - pAY_x01[2,1];
	imeffs0_x10[i] 	<- pAY_x10[2,1] - pAY_x10[3,1];
	imeffs0_x00[i] 	<- as.real(gamma0 > 0) - pAY_x00[3,1] - pAY_x00[4,1];
	imeffs0_x11[i] 	<- pAY_x11[3,1] + pAY_x11[4,1] - as.real(gamma0 < 0);
	py_x00[i] 	<- pAY_x00[3,1] + pAY_x00[4,1];
	py_x11[i] 	<- pAY_x11[3,1] + pAY_x11[4,1];


	# LINEAR PROGRAMMING: BOUNDS FOR PARAMETER AND MARGINAL EFFECT;

	f.rhs     <- t(cbind(t(pAY_x00), t(pAY_x01),t(pAY_x10),t(pAY_x11),t(rep(1, nAX)), t(rep(0, nAX*(1+nAY+M)))));

	for (j in 1:gridgamma) {;

		gamma	  <- gammas[j];

		piAY_x00  <- PIAY(AY, t(AX[1,]), gamma, alphas);
		piAY_x01  <- PIAY(AY, t(AX[3,]), gamma, alphas);
		piAY_x10  <- PIAY(AY, t(AX[2,]), gamma, alphas);
		piAY_x11  <- PIAY(AY, t(AX[4,]), gamma, alphas);

		a1_x00    <- cbind(matrix(0, nAY, nAX), diag(rep(1,nAY)), matrix(0, nAY, 3*nAY), piAY_x00, matrix(0, nAY, 3*M));
		a1_x01    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, nAY), diag(rep(1,nAY)), matrix(0, nAY, 2*nAY),  matrix(0, nAY, M), piAY_x01, matrix(0, nAY, 2*M));
		a1_x10    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 2*nAY), diag(rep(1,nAY)), matrix(0, nAY, nAY),  matrix(0, nAY, 2*M), piAY_x10, matrix(0, nAY, M));
		a1_x11    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 3*nAY), diag(rep(1,nAY)), matrix(0, nAY, 3*M), piAY_x11);
		a2_x00    <- cbind(matrix(c(1,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(1, M)),  t(rep(0, 3*M)));
		a2_x01    <- cbind(matrix(c(0,1,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, M)), t(rep(1, M)),  t(rep(0, 2*M)));
		a2_x10    <- cbind(matrix(c(0,0,1,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 2*M)), t(rep(1, M)),  t(rep(0, M)));
		a2_x11    <- cbind(matrix(c(0,0,0,1),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 3*M)), t(rep(1, M)));
	        a3        <- diag(rep(1, nAX*(1 + nAY + M)));
        	f.con     <- rbind(a1_x00, a1_x01,a1_x10,a1_x11, a2_x00, a2_x01, a2_x10, a2_x11, a3);

		LP        <- lp("min", f.obj, f.con, f.dir, f.rhs);
		OBJ[i,j]  <- LP$objval;
		sol       <- matrix(LP$solution, ncol = 1);


	## SEMIPARAMETRIC BOUNDS FOR MARGINAL EFFECTS

		if (OBJ[i,j] == 0) {;
			meffs_x00    <- ME(gamma, alphas, t(AX[1,]));
			meffs_x11    <- ME(gamma, alphas, t(AX[4,]));

		## x = (0,0);

			fmeff_x00.obj   <- meffs_x00;
			fmeff_x00.dir   <- f_x.dir;
			fmeff_x00.rhs  	<- t(cbind(t(pAY_x00),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x00.con	<- rbind(piAY_x00, t(rep(1, M)), diag(1, M));

			LPlmeff_x00     <- lp("min", fmeff_x00.obj, fmeff_x00.con, fmeff_x00.dir, fmeff_x00.rhs);
			lprhos_x00      <- matrix(LPlmeff_x00$solution, ncol = 1);

			if (LPlmeff_x00$status == 0) {; 
				lmeffs_x00_sp[i,j]	<- LPlmeff_x00$objval;
				};
				
			if (LPlmeff_x00$status == 2) {;
				lmeffs_x00_sp[i,j]	<- NaN;
				lnosol_x00             	<- lnosol_x00 + 1;
				};

			LPumeff_x00   <- lp("max", fmeff_x00.obj, fmeff_x00.con, fmeff_x00.dir, fmeff_x00.rhs);
			uprhos_x00    <- matrix(LPumeff_x00$solution, ncol = 1);
			if (LPumeff_x00$status == 0) {; 
				umeffs_x00_sp[i,j]	<- LPumeff_x00$objval;
				};
				
			if (LPumeff_x00$status == 2) {;
				umeffs_x00_sp[i,j]	<- NaN;
				unosol_x00             	<- unosol_x00 + 1;
				};


		## x = (1,1);

			fmeff_x11.obj   <- meffs_x11;
			fmeff_x11.dir   <- f_x.dir;
			fmeff_x11.rhs  	<- t(cbind(t(pAY_x11),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x11.con	<- rbind(piAY_x11, t(rep(1, M)), diag(1, M));

			LPlmeff_x11     <- lp("min", fmeff_x11.obj, fmeff_x11.con, fmeff_x11.dir, fmeff_x11.rhs);
			lprhos_x11      <- matrix(LPlmeff_x11$solution, ncol = 1);

			if (LPlmeff_x11$status == 0) {; 
				lmeffs_x11_sp[i,j]	<- LPlmeff_x11$objval;
				};
				
			if (LPlmeff_x11$status == 2) {;
				lmeffs_x11_sp[i,j]	<- NaN;
				lnosol_x11             	<- lnosol_x11 + 1;
				};

			LPumeff_x11   <- lp("max", fmeff_x11.obj, fmeff_x11.con, fmeff_x11.dir, fmeff_x11.rhs);
			uprhos_x11    <- matrix(LPumeff_x11$solution, ncol = 1);
			if (LPumeff_x11$status == 0) {; 
				umeffs_x11_sp[i,j]	<- LPumeff_x11$objval;
				};
				
			if (LPumeff_x11$status == 2) {;
				umeffs_x11_sp[i,j]	<- NaN;
				unosol_x11             	<- unosol_x11 + 1;
				};

			};

		};
	};

unosol
lnosol

lnosol_x00;
unosol_x00;
lnosol_x11;
unosol_x11;


umeff_x00_sp  <- apply(umeffs_x00_sp,1,max);
umeff_x01_sp  <- imeffs0_x01;
umeff_x10_sp  <- imeffs0_x10;
umeff_x11_sp  <- apply(umeffs_x11_sp,1,max);
umeff_sp	 <- pAX[1,1]*umeff_x00_sp + pAX[2,1]*umeff_x01_sp + pAX[3,1]*umeff_x10_sp + pAX[4,1]*umeff_x11_sp;

lmeff_x00_sp  <- apply(lmeffs_x00_sp,1,min);
lmeff_x01_sp  <- imeffs0_x01;
lmeff_x10_sp  <- imeffs0_x10;
lmeff_x11_sp  <- apply(lmeffs_x11_sp,1,min);
lmeff_sp	 <- pAX[1,1]*lmeff_x00_sp + pAX[2,1]*lmeff_x01_sp + pAX[3,1]*lmeff_x10_sp + pAX[4,1]*lmeff_x11_sp;


umeff_np	<- pAX[1,1]*(1 - py_x00) + pAX[2,1]*imeffs0_x01 + pAX[3,1]*imeffs0_x10 + pAX[4,1]*py_x11;
lmeff_np	<- pAX[1,1]*(0 - py_x00) + pAX[2,1]*imeffs0_x01 + pAX[3,1]*imeffs0_x10 + pAX[4,1]*(py_x11 - 1);

umeff_np.m	<- pAX[1,1]*(as.real(gammas0 > 0) * imeffs0_x00) + pAX[2,1]*imeffs0_x01 + pAX[3,1]*imeffs0_x10 + pAX[4,1]*(as.real(gammas0 > 0) * imeffs0_x11);
lmeff_np.m	<- pAX[1,1]*(as.real(gammas0 < 0) * imeffs0_x00) + pAX[2,1]*imeffs0_x01 + pAX[3,1]*imeffs0_x10 + pAX[4,1]*(as.real(gammas0 < 0) * imeffs0_x11);


# FIGURES:


postscript("logit-QP-T2.eps",horizontal=FALSE,onefile=FALSE,height=7,width=7,pointsize=10);

par(mfrow=c(2,2));

ugammas_sp   <- matrix(-999, gridgamma0, gridgamma);
lgammas_sp   <- matrix(999, gridgamma0, gridgamma);

plot(range(gammas0), range(gammas0),  type="n",xlab=expression(paste(beta, '*')), ylab='B', main="");
title(main="Parameter",cex.main=1);
	for (i in 1:gridgamma0) {;
		gamma0 <- gammas0[i];
		lgamma <- wgrid * (gamma0 - wgamma) / wgamma;
		ugamma <- wgrid * (gamma0 + wgamma) / wgamma;
		gammas <- wgamma * (lgamma:ugamma) / wgrid;
		if (gridgamma == 1) gammas <- gammas0[i];
		for (j in 1:gridgamma) {;
			if ( (OBJ[i,j] == 0) & (gammas[j] >= gammas0[i] - 0.001)) ugammas_sp[i,j] <- gammas[j]; 
			if ( (OBJ[i,j] == 0) & (gammas[j] <= gammas0[i] + 0.001)) lgammas_sp[i,j] <- gammas[j]; 
			};
		};

	ugamma_sp <- apply(ugammas_sp,1,max);
	lgamma_sp <- apply(lgammas_sp,1,min);
	ugamma_asp <- apply(ugammas_sp,1,max);
	lgamma_asp <- apply(lgammas_sp,1,min);

        polygon(c(gammas0,rev(gammas0)),c(ugamma_sp,rev(lgamma_sp)),density=-100, border = 'grey70', col = 'grey70', lty = 1, lwd = 1);

	lines(gammas0, ugamma_sp, lty = 1, col = 'grey70', lwd = 3)


	lines(gammas0, gammas0,  lty = 1, col = 1);	

	legend(min(gammas0),max(gammas0),c("True","SP-Bound"), lwd=c(1,4), lty = c(1,1), bty="n",col=c(1,'grey70'));



plot( c(min(gammas0), max(gammas0)), c(-.75, .75), type="n",xlab=expression(paste(beta, '*')), ylab=expression(Delta^k), main="");
title(main=expression(paste(X^k, ' = (0,0)')),cex.main=1);


        polygon(c(gammas0,rev(gammas0)),c(umeff_x00_sp,rev(lmeff_x00_sp)), density=-100, border = 'grey70', col = 'grey70', lty = 1, lwd = 1);

	lines(gammas0, meffs0_x00, col=1, lty = 1);

	lines(gammas0, imeffs0_x00, lty = 3, col = 1);	
	lines(gammas0, 0*imeffs0_x00, lty = 3, col = 1);

	legend(min(gammas0),.75,c("True","NPM-Bound","SP-Bound"), lwd = c(1,1,4), lty = c(1,3,1), bty="n",col=c(1,1,'grey70'));



plot( c(min(gammas0), max(gammas0)), c(-.75, .75), type="n",xlab=expression(paste(beta, '*')), ylab=expression(Delta^k), main="");
title(main=expression(paste(X^k, ' = (1,1)')),cex.main=1);


        polygon(c(gammas0,rev(gammas0)),c(umeff_x11_sp,rev(lmeff_x11_sp)), density=-100, border = 'grey70', col = 'grey70', lty = 1, lwd = 1);

	lines(gammas0, meffs0_x11, col=1, lty = 1);

	lines(gammas0, imeffs0_x11, lty = 3, col = 1);	
	lines(gammas0, 0*imeffs0_x11, lty = 3, col = 1);

	legend(min(gammas0),.75,c("True","NPM-Bound","SP-Bound"), lwd = c(1,1,4), lty = c(1,3,1), bty="n",col=c(1,1,'grey70'));



plot( c(min(gammas0), max(gammas0)), c(-.75,.75), type="n",xlab=expression(paste(beta, '*')), ylab=expression(Delta), main="");
title(main="ATE",cex.main=1);

        polygon(c(gammas0,rev(gammas0)),c(umeff_sp,rev(lmeff_sp)), density=-100, border = 'grey70', col = 'grey70', lty = 1, lwd = 1);

	lines(gammas0, meffs00, col=1, lty = 1);

	lines(gammas0, umeff_np, lty = 2, col = 1);	
	lines(gammas0, lmeff_np, lty = 2, col = 1);

	lines(gammas0, umeff_np.m, lty = 3, col = 1);	
	lines(gammas0, lmeff_np.m, lty = 3, col = 1);

	legend(min(gammas0),.75,c("True","NP-Bound", "NPM-Bound", "SP-Bound"), lwd = c(1,1,1,4), lty = c(1,2,3,1), bty="n",col=c(1,1,1,'grey70'));


dev.off();



##############;
# T = 3;
##############;


# PARAMETERS FOR SIMULATION;

nu	<- 0;
T	<- 3;
N	<- 5000000;
nu	<- 0;



A      <- t(cbind(c(0,0,0), c(0,0,1), c(0,1,0), c(1,0,0), c(0,1,1), c(1,0,1), c(1,1,0), c(1,1,1)));
AY      <- A;
AX      <- A;
nAY	<- nrow(AY);
nAX  	<- nrow(AX);

pAX	<- NULL;
for (i in 1:nAX) {;
	pAX	<- rbind(pAX, integrate(probx, -Inf, Inf, T = T, vecx = AX[i, ])$value);
	};

alpha0s	<- c(-Inf,4*(-10:10)/10,Inf);
alpha0s	<- c(-Inf,2*(-20:20)/10,Inf);
alpha0s <- qlogis(c(0:50)/50, location = 0, scale = sqrt(3)/pi);
alpha0s <- qlogis(c(0:100)/100, location = 0, scale = sqrt(3)/pi);
alphas 	<- c(-Inf,2*(-80:80)/10,Inf);
alphas 	<- qlogis(c(0:150)/150, location = 0, scale = sqrt(3)/pi);
alphas 	<- qlogis(c(0:100)/100, location = 0, scale = sqrt(3)/pi);
M      	<- length(alphas);

A      <- t(cbind(c(0,0,0), c(0,0,1), c(0,1,0), c(1,0,0), c(0,1,1), c(1,0,1), c(1,1,0), c(1,1,1)));
AY      <- A;
AX      <- A;
nAY	<- nrow(AY);
nAX  	<- nrow(AX);

# Parameters of the optimization

gridgamma0 <- 201;
ugamma0    <- (gridgamma0 - 1)/2;
gammas0    <- 2*2*(-ugamma0:ugamma0)/(gridgamma0-1);

gridgamma <- 201;
wgamma    <- .5;
wgrid     <- (gridgamma - 1)/2;


# Initialization of variables

OBJ    		<- matrix(-999, gridgamma0, gridgamma);

meffs00    	<- NULL;
meffs0    	<- NULL;
meffs0_x000	<- NULL;
meffs0_x001	<- NULL;
meffs0_x010	<- NULL;
meffs0_x100	<- NULL;
meffs0_x011	<- NULL;
meffs0_x101	<- NULL;
meffs0_x110	<- NULL;
meffs0_x111	<- NULL;

imeffs0_x000	<- NULL;
imeffs0_x001	<- NULL;
imeffs0_x010	<- NULL;
imeffs0_x100	<- NULL;
imeffs0_x011	<- NULL;
imeffs0_x101	<- NULL;
imeffs0_x110	<- NULL;
imeffs0_x111	<- NULL;

py_x000		<- NULL;
py_x111		<- NULL;


meffs_sp    	<- matrix(-999, gridgamma0, gridgamma);

umeffs_x000_sp <- matrix(-999, gridgamma0, gridgamma);
umeffs_x111_sp <- matrix(-999, gridgamma0, gridgamma);

lmeffs_x000_sp <- matrix(999, gridgamma0, gridgamma);
lmeffs_x111_sp <- matrix(999, gridgamma0, gridgamma);

gammas1	<- NULL;

f.obj 	<- t(cbind(t(rep(1,nAY*nAX+nAX)),t(rep(0,nAX*M))));
f.dir 	<- t(cbind(t(rep("=",nAY*nAX+nAX)),t(rep(">=", nAX*(nAY+1+M)))));
f_x.dir <- t(cbind(t(rep("=",nAY+1)),t(rep(">=", M))));

lnosol 	<- 0;
unosol 	<- 0;

lnosol_x000 <- 0;
unosol_x000 <- 0;
lnosol_x111 <- 0;
unosol_x111 <- 0;

for (i in 1:gridgamma0) {;
	gamma0 <- gammas0[i];
	lgamma <- wgrid * (gamma0 - wgamma) / wgamma;
	ugamma <- wgrid * (gamma0 + wgamma) / wgamma;
	gammas <- wgamma * (lgamma:ugamma) / wgrid;

	# TRUE CHOICE PROBABILITIES;

	pAY_x000	<- rbind(integrate(py000x000, -Inf, Inf, beta=gamma0)$value, integrate(py001x000, -Inf, Inf, beta=gamma0)$value, integrate(py010x000, -Inf, Inf, beta=gamma0)$value, integrate(py100x000, -Inf, Inf, beta=gamma0)$value, integrate(py011x000, -Inf, Inf, beta=gamma0)$value, integrate(py101x000, -Inf, Inf, beta=gamma0)$value, integrate(py110x000, -Inf, Inf, beta=gamma0)$value, integrate(py111x000, -Inf, Inf, beta=gamma0)$value)/pAX[1,];
	pAY_x001	<- rbind(integrate(py000x001, -Inf, Inf, beta=gamma0)$value, integrate(py001x001, -Inf, Inf, beta=gamma0)$value, integrate(py010x001, -Inf, Inf, beta=gamma0)$value, integrate(py100x001, -Inf, Inf, beta=gamma0)$value, integrate(py011x001, -Inf, Inf, beta=gamma0)$value, integrate(py101x001, -Inf, Inf, beta=gamma0)$value, integrate(py110x001, -Inf, Inf, beta=gamma0)$value, integrate(py111x001, -Inf, Inf, beta=gamma0)$value)/pAX[2,];
	pAY_x010	<- rbind(integrate(py000x010, -Inf, Inf, beta=gamma0)$value, integrate(py001x010, -Inf, Inf, beta=gamma0)$value, integrate(py010x010, -Inf, Inf, beta=gamma0)$value, integrate(py100x010, -Inf, Inf, beta=gamma0)$value, integrate(py011x010, -Inf, Inf, beta=gamma0)$value, integrate(py101x010, -Inf, Inf, beta=gamma0)$value, integrate(py110x010, -Inf, Inf, beta=gamma0)$value, integrate(py111x010, -Inf, Inf, beta=gamma0)$value)/pAX[3,];
	pAY_x100	<- rbind(integrate(py000x100, -Inf, Inf, beta=gamma0)$value, integrate(py001x100, -Inf, Inf, beta=gamma0)$value, integrate(py010x100, -Inf, Inf, beta=gamma0)$value, integrate(py100x100, -Inf, Inf, beta=gamma0)$value, integrate(py011x100, -Inf, Inf, beta=gamma0)$value, integrate(py101x100, -Inf, Inf, beta=gamma0)$value, integrate(py110x100, -Inf, Inf, beta=gamma0)$value, integrate(py111x100, -Inf, Inf, beta=gamma0)$value)/pAX[4,];
	pAY_x011	<- rbind(integrate(py000x011, -Inf, Inf, beta=gamma0)$value, integrate(py001x011, -Inf, Inf, beta=gamma0)$value, integrate(py010x011, -Inf, Inf, beta=gamma0)$value, integrate(py100x011, -Inf, Inf, beta=gamma0)$value, integrate(py011x011, -Inf, Inf, beta=gamma0)$value, integrate(py101x011, -Inf, Inf, beta=gamma0)$value, integrate(py110x011, -Inf, Inf, beta=gamma0)$value, integrate(py111x011, -Inf, Inf, beta=gamma0)$value)/pAX[5,];
	pAY_x101	<- rbind(integrate(py000x101, -Inf, Inf, beta=gamma0)$value, integrate(py001x101, -Inf, Inf, beta=gamma0)$value, integrate(py010x101, -Inf, Inf, beta=gamma0)$value, integrate(py100x101, -Inf, Inf, beta=gamma0)$value, integrate(py011x101, -Inf, Inf, beta=gamma0)$value, integrate(py101x101, -Inf, Inf, beta=gamma0)$value, integrate(py110x101, -Inf, Inf, beta=gamma0)$value, integrate(py111x101, -Inf, Inf, beta=gamma0)$value)/pAX[6,];
	pAY_x110	<- rbind(integrate(py000x110, -Inf, Inf, beta=gamma0)$value, integrate(py001x110, -Inf, Inf, beta=gamma0)$value, integrate(py010x110, -Inf, Inf, beta=gamma0)$value, integrate(py100x110, -Inf, Inf, beta=gamma0)$value, integrate(py011x110, -Inf, Inf, beta=gamma0)$value, integrate(py101x110, -Inf, Inf, beta=gamma0)$value, integrate(py110x110, -Inf, Inf, beta=gamma0)$value, integrate(py111x110, -Inf, Inf, beta=gamma0)$value)/pAX[7,];
	pAY_x111	<- rbind(integrate(py000x111, -Inf, Inf, beta=gamma0)$value, integrate(py001x111, -Inf, Inf, beta=gamma0)$value, integrate(py010x111, -Inf, Inf, beta=gamma0)$value, integrate(py100x111, -Inf, Inf, beta=gamma0)$value, integrate(py011x111, -Inf, Inf, beta=gamma0)$value, integrate(py101x111, -Inf, Inf, beta=gamma0)$value, integrate(py110x111, -Inf, Inf, beta=gamma0)$value, integrate(py111x111, -Inf, Inf, beta=gamma0)$value)/pAX[8,];



	# TRUE AVERAGE EFFECT;
	meffs00[i]	<- integrate(meffect, -Inf, Inf, gamma=gamma0)$value;
	meffs0_x001[i] <- pAY_x001[2,1] + pAY_x001[5,1] - pAY_x001[4,1] - pAY_x001[7,1];
	meffs0_x010[i] <- pAY_x010[3,1] + pAY_x010[5,1] - pAY_x010[4,1] - pAY_x010[6,1];
	meffs0_x100[i] <- pAY_x100[4,1] + pAY_x100[6,1] - pAY_x100[3,1] - pAY_x100[5,1];
	meffs0_x011[i] <- pAY_x011[2,1] + pAY_x011[5,1] - pAY_x011[4,1] - pAY_x011[7,1];
	meffs0_x101[i] <- pAY_x101[4,1] + pAY_x101[6,1] - pAY_x101[3,1] - pAY_x101[5,1];
	meffs0_x110[i] <- pAY_x110[4,1] + pAY_x110[7,1] - pAY_x110[2,1] - pAY_x110[5,1];
	meffs0_x000[i] 	<- integrate(meffectx0, -Inf, Inf, gamma=gamma0, T=T)$value / pAX[1,];
	meffs0_x111[i] 	<- integrate(meffectx1, -Inf, Inf, gamma=gamma0, T=T)$value / pAX[nAX,];

	# BOUNDS FOR THE PARAMETER;

	# QUADRATIC PROGRAM: REGULARIZATION OF PROBABILITIES TO THE MODEL;

	lambda		<- 1/(N*log(N));

	MM		<- length(alpha0s);

	piAY_x000  	<- PIAY(AY, t(AX[1,]), gamma0, alpha0s);
	piAY_x001  	<- PIAY(AY, t(AX[2,]), gamma0, alpha0s);
	piAY_x010  	<- PIAY(AY, t(AX[3,]), gamma0, alpha0s);
	piAY_x100  	<- PIAY(AY, t(AX[4,]), gamma0, alpha0s);
	piAY_x011  	<- PIAY(AY, t(AX[5,]), gamma0, alpha0s);
	piAY_x101  	<- PIAY(AY, t(AX[6,]), gamma0, alpha0s);
	piAY_x110  	<- PIAY(AY, t(AX[7,]), gamma0, alpha0s);
	piAY_x111  	<- PIAY(AY, t(AX[8,]), gamma0, alpha0s);

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))));

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);


	#REGULARIZED PROBABILITIES;
	pAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	pAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	pAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	pAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	pAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	pAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	pAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	pAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));

	# NONPARARAMETRICALLY IDENTIFIED EFFECTS AND BOUNDS;
	imeffs0_x000[i] <- as.real(gamma0 > 0) - (pAY_x000[2,1] + 2*pAY_x000[5,1] + pAY_x000[8,1]);
	imeffs0_x001[i] <- pAY_x001[2,1] + pAY_x001[5,1] - pAY_x001[4,1] - pAY_x001[7,1];
	imeffs0_x010[i] <- pAY_x010[3,1] + pAY_x010[5,1] - pAY_x010[4,1] - pAY_x010[6,1];
	imeffs0_x100[i] <- pAY_x100[4,1] + pAY_x100[6,1] - pAY_x100[3,1] - pAY_x100[5,1];
	imeffs0_x011[i] <- pAY_x011[2,1] + pAY_x011[5,1] - pAY_x011[4,1] - pAY_x011[7,1];
	imeffs0_x101[i] <- pAY_x101[4,1] + pAY_x101[6,1] - pAY_x101[3,1] - pAY_x101[5,1];
	imeffs0_x110[i] <- pAY_x110[4,1] + pAY_x110[7,1] - pAY_x110[2,1] - pAY_x110[5,1];
	imeffs0_x111[i] <- (pAY_x111[2,1] + 2*pAY_x111[5,1] + pAY_x111[8,1]) - as.real(gamma0 < 0);


	py_x000[i] 	<- pAY_x000[2,1] + 2*pAY_x000[5,1] + pAY_x000[8,1];
	py_x111[i] 	<- pAY_x111[2,1] + 2*pAY_x111[5,1] + pAY_x111[8,1];


	# LINEAR PROGRAMMING: BOUNDS FOR PARAMETER AND MARGINAL EFFECT;

	f.rhs        <- t(cbind(t(pAY_x000), t(pAY_x001),t(pAY_x010),t(pAY_x100),t(pAY_x011),t(pAY_x101),t(pAY_x110),t(pAY_x111),t(rep(1, nAX)), t(rep(0, nAX*(1+nAY+M)))));

#	for (j in 1:gridgamma) {;

		j	<- 1;		
		gamma	<- gamma0;

		piAY_x000  <- PIAY(AY, t(AX[1,]), gamma, alphas);
		piAY_x001  <- PIAY(AY, t(AX[2,]), gamma, alphas);
		piAY_x010  <- PIAY(AY, t(AX[3,]), gamma, alphas);
		piAY_x100  <- PIAY(AY, t(AX[4,]), gamma, alphas);
		piAY_x011  <- PIAY(AY, t(AX[5,]), gamma, alphas);
		piAY_x101  <- PIAY(AY, t(AX[6,]), gamma, alphas);
		piAY_x110  <- PIAY(AY, t(AX[7,]), gamma, alphas);
		piAY_x111  <- PIAY(AY, t(AX[8,]), gamma, alphas);

		a1_x000    <- cbind(matrix(0, nAY, nAX), diag(rep(1,nAY)), matrix(0, nAY, 7*nAY), piAY_x000, matrix(0, nAY, 7*M));
		a1_x001    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, nAY), diag(rep(1,nAY)), matrix(0, nAY, 6*nAY), matrix(0, nAY, M), piAY_x001, matrix(0, nAY, 6*M));
		a1_x010    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 2*nAY), diag(rep(1,nAY)), matrix(0, nAY, 5*nAY), matrix(0, nAY, 2*M), piAY_x010, matrix(0, nAY, 5*M));
		a1_x100    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 3*nAY), diag(rep(1,nAY)), matrix(0, nAY, 4*nAY), matrix(0, nAY, 3*M), piAY_x100, matrix(0, nAY, 4*M));
		a1_x011    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 4*nAY), diag(rep(1,nAY)), matrix(0, nAY, 3*nAY), matrix(0, nAY, 4*M), piAY_x011, matrix(0, nAY, 3*M));
		a1_x101    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 5*nAY), diag(rep(1,nAY)), matrix(0, nAY, 2*nAY), matrix(0, nAY, 5*M), piAY_x101, matrix(0, nAY, 2*M));
		a1_x110    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 6*nAY), diag(rep(1,nAY)), matrix(0, nAY, nAY), matrix(0, nAY, 6*M), piAY_x110, matrix(0, nAY, M));
		a1_x111    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 7*nAY), diag(rep(1,nAY)), matrix(0, nAY, 7*M), piAY_x111);
		a2_x000    <- cbind(matrix(c(1,0,0,0,0,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(1, M)),  t(rep(0, 7*M)));
		a2_x001    <- cbind(matrix(c(0,1,0,0,0,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, M)), t(rep(1, M)),  t(rep(0, 6*M)));
		a2_x010    <- cbind(matrix(c(0,0,1,0,0,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 2*M)), t(rep(1, M)),  t(rep(0, 5*M)));
		a2_x100    <- cbind(matrix(c(0,0,0,1,0,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 3*M)), t(rep(1, M)),  t(rep(0, 4*M)));
		a2_x011    <- cbind(matrix(c(0,0,0,0,1,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 4*M)), t(rep(1, M)),  t(rep(0, 3*M)));
		a2_x101    <- cbind(matrix(c(0,0,0,0,0,1,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 5*M)), t(rep(1, M)),  t(rep(0, 2*M)));
		a2_x110    <- cbind(matrix(c(0,0,0,0,0,0,1,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 6*M)), t(rep(1, M)),  t(rep(0, M)));
		a2_x111    <- cbind(matrix(c(0,0,0,0,0,0,0,1),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 7*M)), t(rep(1, M)));
                a3         <- diag(rep(1, nAX*(1 + nAY + M)));
                f.con      <- rbind(a1_x000, a1_x001,a1_x010,a1_x100,a1_x011,a1_x101,a1_x110,a1_x111,a2_x000, a2_x001, a2_x010, a2_x100, a2_x011, a2_x101, a2_x110, a2_x111, a3);

		LP        <- lp("min", f.obj, f.con, f.dir, f.rhs);
		OBJ[i,j]  <- LP$objval;
		sol       <- matrix(LP$solution, ncol = 1);



	## SEMIPARAMETRIC BOUNDS FOR MARGINAL EFFECTS

		if (OBJ[i,j] == 0) {;
			meffs_x000    <- ME(gamma, alphas, t(AX[1,]));
			meffs_x111    <- ME(gamma, alphas, t(AX[nrow(AX),]));


		## x = (0,0,0);

			fmeff_x000.obj  <- meffs_x000;
			fmeff_x000.dir  <- f_x.dir;
			fmeff_x000.rhs  <- t(cbind(t(pAY_x000),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x000.con	<- rbind(piAY_x000, t(rep(1, M)), diag(1, M));

			LPlmeff_x000     <- lp("min", fmeff_x000.obj, fmeff_x000.con, fmeff_x000.dir, fmeff_x000.rhs);
			lprhos_x000      <- matrix(LPlmeff_x000$solution, ncol = 1);

			if (LPlmeff_x000$status == 0) {; 
				lmeffs_x000_sp[i,j]	<- LPlmeff_x000$objval;
				};
				
			if (LPlmeff_x000$status == 2) {;
				lmeffs_x000_sp[i,j]	<- NaN;
				lnosol_x000             	<- lnosol_x000 + 1;
				};

			LPumeff_x000   <- lp("max", fmeff_x000.obj, fmeff_x000.con, fmeff_x000.dir, fmeff_x000.rhs);
			uprhos_x000    <- matrix(LPumeff_x000$solution, ncol = 1);
			if (LPumeff_x000$status == 0) {; 
				umeffs_x000_sp[i,j]	<- LPumeff_x000$objval;
				};
				
			if (LPumeff_x000$status == 2) {;
				umeffs_x000_sp[i,j]	<- NaN;
				unosol_x000             <- unosol_x000 + 1;
				};


		## x = (1,1,1);

			fmeff_x111.obj   <- meffs_x111;
			fmeff_x111.dir   <- f_x.dir;
			fmeff_x111.rhs  	<- t(cbind(t(pAY_x111),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x111.con	<- rbind(piAY_x111, t(rep(1, M)), diag(1, M));

			LPlmeff_x111     <- lp("min", fmeff_x111.obj, fmeff_x111.con, fmeff_x111.dir, fmeff_x111.rhs);
			lprhos_x111      <- matrix(LPlmeff_x111$solution, ncol = 1);

			if (LPlmeff_x111$status == 0) {; 
				lmeffs_x111_sp[i,j]	<- LPlmeff_x111$objval;
				};
				
			if (LPlmeff_x111$status == 2) {;
				lmeffs_x111_sp[i,j]	<- NaN;
				lnosol_x111             	<- lnosol_x111 + 1;
				};

			LPumeff_x111   <- lp("max", fmeff_x111.obj, fmeff_x111.con, fmeff_x111.dir, fmeff_x111.rhs);
			uprhos_x111    <- matrix(LPumeff_x111$solution, ncol = 1);
			if (LPumeff_x111$status == 0) {; 
				umeffs_x111_sp[i,j]	<- LPumeff_x111$objval;
				};
				
			if (LPumeff_x111$status == 2) {;
				umeffs_x111_sp[i,j]	<- NaN;
				unosol_x111             <- unosol_x111 + 1;
				};



			};
	};

unosol
lnosol

lnosol_x000;
unosol_x000;
lnosol_x111;
unosol_x111;


umeff_x000_sp  	<- apply(umeffs_x000_sp,1,max);
umeff_x001_sp  	<- imeffs0_x001;
umeff_x010_sp  	<- imeffs0_x010;
umeff_x100_sp  	<- imeffs0_x100;
umeff_x011_sp  	<- imeffs0_x011;
umeff_x101_sp  	<- imeffs0_x101;
umeff_x110_sp  	<- imeffs0_x110;
umeff_x111_sp  	<- apply(umeffs_x111_sp,1,max);
umeff_sp	<- pAX[1,1]*umeff_x000_sp + pAX[2,1]*umeff_x001_sp + pAX[3,1]*umeff_x010_sp + pAX[4,1]*umeff_x100_sp + pAX[5,1]*umeff_x011_sp + pAX[6,1]*umeff_x101_sp + pAX[7,1]*umeff_x110_sp + pAX[8,1]*umeff_x111_sp;

lmeff_x000_sp  	<- apply(lmeffs_x000_sp,1,min);
lmeff_x001_sp  	<- imeffs0_x001;
lmeff_x010_sp  	<- imeffs0_x010;
lmeff_x100_sp  	<- imeffs0_x100;
lmeff_x011_sp  	<- imeffs0_x011;
lmeff_x101_sp  	<- imeffs0_x101;
lmeff_x110_sp  	<- imeffs0_x110;
lmeff_x111_sp  	<- apply(lmeffs_x111_sp,1,min);
lmeff_sp	<- pAX[1,1]*lmeff_x000_sp + pAX[2,1]*lmeff_x001_sp + pAX[3,1]*lmeff_x010_sp + pAX[4,1]*lmeff_x100_sp + pAX[5,1]*lmeff_x011_sp + pAX[6,1]*lmeff_x101_sp + pAX[7,1]*lmeff_x110_sp + pAX[8,1]*lmeff_x111_sp;

umeff_np	<- pAX[1,1]*(1 - py_x000) + pAX[2,1]*imeffs0_x001 + pAX[3,1]*imeffs0_x010  + pAX[4,1]*imeffs0_x100 + pAX[5,1]*imeffs0_x011  + pAX[6,1]*imeffs0_x101 + pAX[7,1]*imeffs0_x110 + pAX[8,1]*py_x111;
lmeff_np	<- pAX[1,1]*(0 - py_x000) + pAX[2,1]*imeffs0_x001 + pAX[3,1]*imeffs0_x010  + pAX[4,1]*imeffs0_x100 + pAX[5,1]*imeffs0_x011  + pAX[6,1]*imeffs0_x101 + pAX[7,1]*imeffs0_x110 + pAX[8,1]*(py_x111 - 1);

umeff_np.m	<- pAX[1,1]*(as.real(gammas0 > 0) * imeffs0_x000) + pAX[2,1]*imeffs0_x001 + pAX[3,1]*imeffs0_x010  + pAX[4,1]*imeffs0_x100 + pAX[5,1]*imeffs0_x011  + pAX[6,1]*imeffs0_x101 + pAX[7,1]*imeffs0_x110 + pAX[8,1]*(as.real(gammas0 > 0) * imeffs0_x111);
lmeff_np.m	<- pAX[1,1]*(as.real(gammas0 < 0) * imeffs0_x000) + pAX[2,1]*imeffs0_x001 + pAX[3,1]*imeffs0_x010  + pAX[4,1]*imeffs0_x100 + pAX[5,1]*imeffs0_x011  + pAX[6,1]*imeffs0_x101 + pAX[7,1]*imeffs0_x110 + pAX[8,1]*(as.real(gammas0 < 0) * imeffs0_x111);


# FIGURES:


postscript("logit-QP-T3.eps",horizontal=FALSE,onefile=FALSE,height=7,width=7,pointsize=10);

par(mfrow=c(2,2));

plot(range(gammas0), range(gammas0),  type="n",xlab=expression(paste(beta, '*')), ylab='B', main="");
title(main="Parameter",cex.main=1);

	lines(gammas0, gammas0, lty = 1, col = 'grey70', lwd = 3)
	lines(gammas0, gammas0,  lty = 1, col = 1);	

	legend(min(gammas0),max(gammas0),c("True","SP-Bound"), lwd=c(1,4), lty = c(1,1), bty="n",col=c(1,'grey70'));

plot( c(min(gammas0), max(gammas0)), c(-.75, .75), type="n",xlab=expression(paste(beta, '*')), ylab=expression(Delta^k), main="");
title(main=expression(paste(X^k, ' = (0,0,0)')),cex.main=1);


        polygon(c(gammas0,rev(gammas0)),c(umeff_x000_sp,rev(lmeff_x000_sp)), density=-100, border = 'grey70', col = 'grey70', lty = 1, lwd = 1);

	lines(gammas0, meffs0_x000, col=1, lty = 1);

	lines(gammas0, imeffs0_x000, lty = 3, col = 1);	
	lines(gammas0, 0*imeffs0_x000, lty = 3, col = 1);

	legend(min(gammas0),.75,c("True","NPM-Bound","SP-Bound"), lwd = c(1,1,4), lty = c(1,3,1), bty="n",col=c(1,1,'grey70'));



plot( c(min(gammas0), max(gammas0)), c(-.75, .75), type="n",xlab=expression(paste(beta, '*')), ylab=expression(Delta^k), main="");
title(main=expression(paste(X^k, ' = (1,1,1)')),cex.main=1);


        polygon(c(gammas0,rev(gammas0)),c(umeff_x111_sp,rev(lmeff_x111_sp)), density=-100, border = 'grey70', col = 'grey70', lty = 1, lwd = 1);

	lines(gammas0, meffs0_x111, col=1, lty = 1);

	lines(gammas0, imeffs0_x111, lty = 3, col = 1);	
	lines(gammas0, 0*imeffs0_x111, lty = 3, col = 1);

	legend(min(gammas0),.75,c("True","NPM-Bound","SP-Bound"), lwd = c(1,1,4), lty = c(1,3,1), bty="n",col=c(1,1,'grey70'));



plot( c(min(gammas0), max(gammas0)), c(-.75,.75), type="n",xlab=expression(paste(beta, '*')), ylab=expression(Delta), main="");
title(main="ATE",cex.main=1);

        polygon(c(gammas0,rev(gammas0)),c(umeff_sp,rev(lmeff_sp)), density=-100, border = 'grey70', col = 'grey70', lty = 1, lwd = 1);

	lines(gammas0, meffs00, col=1, lty = 1);

	lines(gammas0, umeff_np, lty = 2, col = 1);	
	lines(gammas0, lmeff_np, lty = 2, col = 1);

	lines(gammas0, umeff_np.m, lty = 3, col = 1);	
	lines(gammas0, lmeff_np.m, lty = 3, col = 1);

	legend(min(gammas0),.75,c("True","NP-Bound", "NPM-Bound", "SP-Bound"), lwd = c(1,1,1,4), lty = c(1,2,3,1), bty="n",col=c(1,1,1,'grey70'));


dev.off();


