###########################################################################################################################
## Chernozhukov, Fernandez-Val, Hahn, and Newey: "Average and Quantile Effects in Nonseparable Panel Models"
## R program to generate Table 1
###########################################################################################################################

# DATA: NLSY 1986-1993 (2.065 MEN)

# Variables:
#    1 - y: log-wage in $1980
#    2 - x: union status binary indicator
#    3 - tx: type of individual with respect to history of x using binary coding, 1 + sum_t 2^{x_t}
#    4 - tx1: type of individual with respect to union status in first period, = 1 if x_1 = 1
#    5 - xf1: individual indicator for first period unionized
#    6 - xf0: individual indicator for first period non-unionized
#    7 - xl1: individual indicator for unionized in the last period
#    8 - xl0: individual indicator for non-unionized in the last period

# This version: 08/28/2012

#####################################################
# PART I: PRELIMINARY FUNCTIONS
#####################################################

rm(list = ls());

#####################################################
# PART II: MAIN PROGRAM
#####################################################


library(stats);
library(foreign);

# Reading dataset;
data	<- read.dta("data_static_T8_long.dta");
T	<- 8;
n	<- nrow(data)/T;
B	<- 200;
attach(data);


table0 <- matrix(0, nrow = T, ncol = 7, dimnames = list(c("1986","1987","1988","1989","1990","1991","1992","1993"),c("Mean","Std. Dev.","Q10","Q25","Q50","Q75","Q90")));
table1 <- matrix(0, nrow = T, ncol = 7, dimnames = list(c("1986","1987","1988","1989","1990","1991","1992","1993"),c("Mean","Std. Dev.","Q10","Q25","Q50","Q75","Q90")));
taus   <- c(.1,.25,.5,.75,.9);

for (t in 1:T) {;
    table0[t, ] <- c(mean(y[tx == 1 & time == t]), sd(y[tx == 1 & time == t]), quantile(y[tx == 1 & time == t], taus));            
    table1[t, ] <- c(mean(y[tx == 2^T & time == t]), sd(y[tx == 2^T & time == t]), quantile(y[tx == 2^T & time == t], taus));            
    };

100*mean(tx==1);
table0;
100*mean(tx==2^T);
table1;
